import datetime

from browser import html, document

num_questions_to_answer = 12
answers = list(zip([1, 0, -1], ['', '', '']))

def make_texts():
    def _(x):
        return x
    return {
        'no_result': _('''
                 .
                   ,    .
            '''),
        'against': _(''),
        'for': _(''),
        'unsure': _(' '),
        'skip': _('  '),
        'more_info': _(' '),
        'party': _(''),
        'pos': _(''),
        'answered_one': _(' \n\n'),
        'answered_many': _(' \n%d\n'),
        'question': _(''),
        'questions': _(''),
        'one': _(''),
        'results': _(''),
        'details': _('  ?'),
        'details_share': _('  ! ?'),
        'new_parties_footnote': _('*         .          '),
        'share': _('   !'),
        'scoring': _('  ?'),
        'restart': _('  ( )'),
        'party': _(''),
        'good': _(''),
        'bad': _(''),
        'total': _(''),
        'align_right': _('right'),
        'lang_prefix': _('/'),
        'congrat': _(' !   %d !'),
        'congrat_results': _('      !'),
        'congrat_resume': _('          .'),
        }
texts = make_texts()

def question_panel(data, is_staff, _):
    outer_frame = html.DIV(id='q%d'%data['id'], Class='question-box')
    panel = html.DIV(id='q%d-panel'%data['id'])
    outer_frame <= panel

    against = html.IMG(src='/Scripts/elections2015/img/btn-against.png')
    in_favor = html.IMG(src='/Scripts/elections2015/img/btn-for.png')
    content = html.DIV()
    skip = html.IMG(src='/Scripts/elections2015/img/btn-skip.png', width='109px')
    party_votes_doc = html.DIV(id='q%d_party_votes'%data['id'], Class='table-responsive')

    panel <= content
    vote_buttons = html.DIV(
        Class='vote-buttons', style={'position': 'relative'},
        id='q%d-buttons'%data['id'])
    vote_buttons <= html.LABEL(in_favor, Class='answer answer-button answer-for')
    vote_buttons <= html.LABEL(against, Class='answer answer-button answer-against')
    vote_buttons <= html.DIV(
        html.LABEL(skip, Class='answer answer-skip'),
        style={'text-align': 'center'},
        )
    panel <= vote_buttons
    panel <= party_votes_doc

    panel <= html.DIV(Class='like')
    panel <= html.DIV(Class='dislike')

    against_radio = html.INPUT(type='radio', name=str(data['id']), value='-1')
    skip_radio = html.INPUT(type='radio', name=str(data['id']), value='0')
    for_radio = html.INPUT(type='radio', name=str(data['id']), value='1')

    against <= against_radio
    against <= html.DIV(_(texts['against']))
    against <= html.DIV(
        html.SPAN(Class='glyphicon glyphicon-arrow-left',
            style={'float': 'right', 'padding-right': '2px'}))
    against <= html.DIV(style={'clear': 'both', 'height': '5px'})

    in_favor <= for_radio
    in_favor <= html.DIV(_(texts['for']))
    in_favor <= html.DIV(
        html.SPAN(Class='glyphicon glyphicon-arrow-right',
            style={'float': 'left', 'padding-left': '2px'}))
    in_favor <= html.DIV(style={'clear': 'both', 'height': '5px'})

    skip <= skip_radio
    skip <= html.SPAN(_(texts['unsure'])+'? ')
    skip <= html.SPAN(_(texts['skip']), style={'text-decoration': 'underline'})

    title = data['title']
    if is_staff:
        title = html.A(
            title, target='_blank', href='/en/admin/vote/vote/%d/' % data['id'])
    content <= html.H3(title)

    description = data['summary']
    too_long = 1000
    if description and len(description) > too_long:
        # it is silly to open the tooltip for one extra word..
        cut_at = len(description[:too_long - 400].rsplit(' ', 1)[0])+1
        tooltip = description[cut_at:]
        description = description[:cut_at]+'...'
    else:
        tooltip = None
    if tooltip:
        summary = html.P(**{
            'data-original-title': tooltip,
            'data-toggle': 'tooltip',
            'data-placement': 'bottom',
            'data-html': 'true',
            })
    else:
        summary = html.P()
    if description:
        first_block = True
        for block in description.split('<br>'):
            if not block.strip():
                continue
            if first_block:
                first_block = False
            else:
                summary <= html.BR()
            summary <= block
        summary <= ' '
    if tooltip:
        summary <= html.BR()
    summary <= html.A(
        _(texts['more_info']),
        target='_blank',
        href='https://oknesset.org/vote/%d/' % data['id'],
        )
    content <= summary
    return outer_frame, party_votes_doc, [against_radio, skip_radio, for_radio]

def question_party_votes(party_votes_doc, data, user_answer, _):
    document['q%d-panel'%data['id']].removeChild(
        document['q%d-buttons'%data['id']])
    def key(x):
        results = x[1]
        return x[0]
    table = html.TABLE(
        style={'text-align': 'center', 'background': '#f9f9f9'},
        Class='table table-packed')
    party_votes_doc <= table
    parties_row = html.TR(
        html.TH(_(texts['party']), style={'vertical-align': 'top'}))
    table <= html.THEAD(parties_row)
    tbody = html.TBODY()
    table <= tbody
    rows = {}
    for (v, name) in answers:
        if not v:
            continue
        style = {}
        if user_answer or not rows:
            style['background'] = 'white'
        if v == user_answer:
            style['color'] = '#07f'
            style['font-weight'] = 'bold'
        elif user_answer:
            style['color'] = '#666'
        row = html.TR(html.TD(_(name)), style=style)
        tbody <= row
        rows[v] = row
    for party_name, results in sorted(data['party_votes'].items(), key=key):
        [for_txt, vs_txt] = [
            '%.0f%%'%(100*r) if r else '-'
            for r in [results.get('for'), results.get('against')]]
        for row, val in [
            (parties_row, _(party_name)),
            (rows[1], for_txt),
            (rows[-1], vs_txt),
            ]:
            row <= html.TD(val)

def calc_results(questions, user_answers):
    results = {}
    num_answers = 0
    site_hash = ''
    for qid, answer in user_answers.items():
        if not answer:
            continue
        site_hash += 'q%d%s' % (qid, 'f' if answer == 1 else 'a')
        num_answers += 1
        question = questions[qid]
        for party_name, votes in question['party_votes'].items():
            vote_results = {-1: 0, 1: 0}
            for k, v in votes.items():
                vote_vals = {'for': 1, 'against': -1}
                if k not in vote_vals:
                    continue
                vote_results[vote_vals[k] * answer] += v
            results.setdefault(party_name, []).append(vote_results)
    window.location.hash = site_hash
    for party_name, t in results.items():
        num_answers_where_party_exists = len(t)
        num_answers_without_party = num_answers - num_answers_where_party_exists
        divisor = num_answers_where_party_exists + num_answers_without_party/2
        s = {}
        for k in [-1, 1]:
            s[k] = sum(x[k] for x in t) / divisor
        s['overall'] = s[1] - s[-1]
        results[party_name] = s
    return results, num_answers

def sorted_results(results):
    def key(x):
        return (-x[1]['overall'], x[0])
    prev_score = None
    for idx, (party_name, score) in enumerate(sorted(list(results.items()), key=key)):
        if idx == 0 or score['overall'] < prev_score['overall']:
            pos = idx+1
        prev_score = score
        yield pos, party_name, score

party_links = {
    '': 'https://www.likud.org.il/',
    ' ': 'http://www.hamahanehazioni.co.il/',
    ' ': 'http://yeshatid.org.il/',
    ' ': 'http://www.beytenu.org.il/',
    ' ': 'http://www.baityehudi.org.il',
    '': 'http://shas.org.il/',
    ' ': 'https://he.wikipedia.org/wiki/_',
    '': 'http://meretz.org.il/',
    ' ': 'https://he.wikipedia.org/wiki/_',
    '': 'https://www.kulanu-party.co.il/',
    '': 'https://he.wikipedia.org/wiki/___',
    ' ': 'https://he.wikipedia.org/wiki/_',
    }

def render_results_table(results, _):
    results_table = html.TABLE(Class='table')
    header_row = html.TR()
    header_row <= html.TH(_(texts['pos']))
    header_row <= html.TH(_(texts['party']))
    header_row <= html.TH(_(texts['good']))
    header_row <= html.TH(_(texts['bad']))
    header_row <= html.TH(_(texts['total']))
    results_table <= html.THEAD(header_row)
    table_body = html.TBODY()
    results_table <= table_body
    if not results:
        table_body <= html.TR(html.TD(_(texts['no_result']), colspan=5))
        return results_table
    for pos, party_name, score in sorted_results(results):
        row = html.TR()
        row <= html.TD(str(pos))
        link = party_links.get(party_name.strip('*'))
        if link:
            party_name = html.A(_(party_name), target='_blank', href=link)
        row <= html.TD(party_name)
        for k in [1, -1, 'overall']:
            cell = '%.0f%%'%(100*score[k])
            row <= html.TD(cell, dir='ltr')
        table_body <= row
    return results_table

def render_results(results_dest, results_small_dest, progress_dest, progress_circle_dest, res, user_answers, _):
    (results, num_answers) = res

    progress_text_lines = '  %d/%d ' % (
        num_answers, num_questions_to_answer)
    if num_answers >= num_questions_to_answer:
        document['bottom'].clear()
        document['questions'].class_name = 'hidden'
        document['results-title'].clear()
        document['results-title'] <= ''
    progress = min(1, num_answers/num_questions_to_answer)
    progress_dest <= html.DIV(
        progress_text_lines.replace('\n', ' '),
        id='progress-text')
    progress_bar = html.DIV(Class='progress')
    progress_bar <= html.DIV(
        html.DIV(
            Class='progress-bar',
            role='progressbar',
            style={
                'width': '%d%%' % (100*progress),
                'min-width': '2em',
                'float': _(texts['align_right']),
                }),
        Class='progress')
    progress_bar <= html.DIV(Class='progress-overlay')
    progress_dest <= progress_bar

    results_dest <= render_results_table(results, _)

    progress_rotate = 'rotate(%ddeg)' % (180+360*progress)
    circle_fill_style = {
        'transform': progress_rotate,
        '-webkit-transform': progress_rotate,
        '-ms-transform': progress_rotate,
        'background-color': '#eee',
        }
    radial_progress_style = {}
    progress_color = '#fd7f19'
    if progress <= 0.5:
        circle_fill_style['background-color'] = progress_color
        progress_clip = 'rect(0px, 150px, 150px, 75px)'
    else:
        radial_progress_style['background-color'] = progress_color
        progress_clip = 'rect(0px, 75px, 150px, 0px)'
    circle_fill_style['clip'] = progress_clip
    radial_progress = html.DIV(Class='radial-progress', style=radial_progress_style)
    progress_circle_dest <= radial_progress
    circle_fill = html.DIV(Class='fill', style=circle_fill_style)
    radial_progress <= html.DIV(
        html.DIV(circle_fill, Class='mask', style={'clip': progress_clip}),
        Class='circle')
    radial_progress_inset = html.DIV(Class='inset', style={'color': progress_color})
    for i, t in enumerate(progress_text_lines.split('\n')):
        if i > 0:
            radial_progress_inset <= html.BR()
        radial_progress_inset <= html.B(t)
    radial_progress <= radial_progress_inset

    if not results:
        return

    if progress >= 1:
        results_small_style = {'color': 'black'}
    else:
        x = 255 * (1-progress)
        results_small_style={
            'color': 'rgba(0,0,0,0.0)',
            'text-shadow': '0 0 %fpx rgb(%d,%d,%d)' %
                (4 * (1-progress), x, x, x),
            }
    results_small = html.DIV(style=results_small_style)
    results_small_dest <= results_small
    results_small <= html.B(_(texts['results'])+':')

    for pos, party_name, score in sorted_results(results):
        results_small <= html.BR()
        results_small <= '%d. %s' % (pos, _(party_name))

    results_small <= html.BR()
    if progress < 1:
        results_small <= _(texts['details'])
    else:
        results_small <= _(texts['details_share'])

    results_dest <= html.A(
        _(texts['scoring']), target='_blank',
        href='scoring.html', style={'float': 'left'}, Class='scoring')
    results_dest <= html.DIV(style={'clear': 'both'})

    results_dest <= html.DIV(
        _(texts['new_parties_footnote']), Class='new-parties')

    bottom_buttons = html.DIV(style={
        'text-align': 'center',
        'position': 'absolute',
        'margin-left': 'auto',
        'margin-right': 'auto',
        'left': '0',
        'right': '0',
        })
    if num_answers >= num_questions_to_answer:
        votes_str = ''.join(
            'q%d%s'%(k, 'f' if v == 1 else 'a') for k, v in sorted(user_answers.items()) if v)
        bottom_buttons <= html.BUTTON(
            html.IMG(src='/Scripts/elections2015/img/btn-share.png', width='180px'),
            onclick='facebook_share()',
            style={'background': 'none', 'outline': 0, 'border': 0})

    bottom_buttons <= html.A(html.IMG(src='/Scripts/elections2015/img/btn-restart.png', width='114px'), href='/Scripts/elections2015/index.html')

    results_dest <= html.DIV(
        bottom_buttons, style={'text-align': 'center', 'position': 'relative'})

from browser import window
window.num_questions_to_answer = num_questions_to_answer
window.texts = texts
window.render_content = {
    'question_panel': question_panel,
    'question_party_votes': question_party_votes,
    'calc_results': calc_results,
    'render_results': render_results,
    }
